import _toConsumableArray from 'babel-runtime/helpers/toConsumableArray';
import _defineProperty from 'babel-runtime/helpers/defineProperty';
import _Object$entries from 'babel-runtime/core-js/object/entries';
import _getIterator from 'babel-runtime/core-js/get-iterator';
import _slicedToArray from 'babel-runtime/helpers/slicedToArray';
import _Object$assign from 'babel-runtime/core-js/object/assign';
import camelCase from 'redux-actions/lib/camelCase';
import { createAction, handleActions } from 'redux-actions';
import { select } from 'redux-saga/effects';
import { createSelector, defaultMemoize, createSelectorCreator, createStructuredSelector } from 'reselect';
import { isForkEffect, isGeneratorFunction, isNormalFunction, mapValues } from './utils';
import { enhance as _enhance, enhancibleForkerThunks, enhancedForkers } from './enhance';

var initializeSelectors = function initializeSelectors(moduleName, factory) {
  var selectors = { selectModule: function selectModule(state) {
      return state[moduleName];
    } };
  if (factory && !isNormalFunction(factory)) {
    throw new Error('Invalid option selectorFactory: It must be a function that returns selector definitions.');
  }
  if (isNormalFunction(factory)) {
    _Object$assign(selectors, factory({ selectModule: selectors.selectModule, createSelector: createSelector, defaultMemoize: defaultMemoize, createSelectorCreator: createSelectorCreator, createStructuredSelector: createStructuredSelector }));
  }

  var _loop = function _loop(name, fn) {
    if (!isNormalFunction(fn)) throw new Error('Invalid selector entry ' + name + ': It must be a function.');
    fn.effect = function () {
      for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      return select.apply(undefined, [fn].concat(args));
    };
  };

  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = _getIterator(_Object$entries(selectors)), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var _ref = _step.value;

      var _ref2 = _slicedToArray(_ref, 2);

      var name = _ref2[0];
      var fn = _ref2[1];

      _loop(name, fn);
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  return selectors;
};

var thunkifyMainGeneratorFunction = function thunkifyMainGeneratorFunction(saga) {
  return function (_ref3) {
    var type = _ref3.type,
        takeEvery = _ref3.takeEvery;
    return takeEvery(type, saga);
  };
};
var thunkifyAdditionalGeneratorFunction = function thunkifyAdditionalGeneratorFunction(saga) {
  return function (_ref4) {
    var fork = _ref4.fork;
    return fork(saga);
  };
};

var initializeMainSaga = function initializeMainSaga(saga, selectors, type, onError) {
  var errLabels = ['Invalid saga for ' + type, 'takeEvery'];
  return initializeSaga(thunkifyMainGeneratorFunction, saga, selectors, errLabels, { type: type }, onError);
};
var initializeAdditionalSaga = function initializeAdditionalSaga(saga, selectors, types, name) {
  var errLabels = ['Invalid additional saga ' + name, 'fork'];
  return initializeSaga(thunkifyAdditionalGeneratorFunction, saga, selectors, errLabels, types);
};

var initializeSaga = function initializeSaga(thunkify, saga, selectors, errLabels, types, onError) {
  if (typeof saga === 'function') {
    saga = (isGeneratorFunction(saga) ? thunkify(saga) : saga)(_Object$assign({}, types, mapValues(enhancibleForkerThunks, function (thunk) {
      return thunk(onError);
    }), {
      enhance: function enhance(saga) {
        return _enhance(saga, onError);
      }
    }, selectors));
  }
  if (isGeneratorFunction(saga)) return enhancedForkers.fork(saga);
  if (isForkEffect(saga)) return saga;
  throw new Error(errLabels[0] + ': It must be specified as one of them: \n' + ('  - generator function (=> automatically invoked by enhanced ' + errLabels[1] + '())\n') + '  - thunk that returns generator function (=> automatically invoked by enhanced fork())\n' + '  - thunk that returns redux-saga FORK effect\n' + '  - redux-saga FORK effect');
};

var createModuleWithApp = function createModuleWithApp(moduleName, definitions, initialState, options, appName) {
  var _Object$assign2;

  var prefix = appName ? '@@' + appName + '/' : '';
  var reducerMap = {};
  var actions = {};
  var actionCreators = {};
  var sagas = {};
  var selectors = initializeSelectors(moduleName, options.selectorFactory);

  var _iteratorNormalCompletion2 = true;
  var _didIteratorError2 = false;
  var _iteratorError2 = undefined;

  try {
    for (var _iterator2 = _getIterator(_Object$entries(definitions)), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
      var _ref5 = _step2.value;

      var _ref6 = _slicedToArray(_ref5, 2);

      var type = _ref6[0];
      var definition = _ref6[1];


      var actionType = '' + prefix + moduleName + '/' + type;
      var camelType = camelCase(type);

      var _ref9 = typeof definition === 'function' ? { reducer: definition } : definition,
          creator = _ref9.creator,
          reducer = _ref9.reducer,
          saga = _ref9.saga,
          onError = _ref9.onError;

      reducer && (reducerMap[actionType] = reducer);
      actions[type] = actionType;
      actionCreators[camelType] = createAction.apply(undefined, [actionType].concat(_toConsumableArray(Array.isArray(creator) ? creator : [creator])));
      saga && (sagas[camelType] = initializeMainSaga(saga, selectors, actionType, onError));
    }
  } catch (err) {
    _didIteratorError2 = true;
    _iteratorError2 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion2 && _iterator2.return) {
        _iterator2.return();
      }
    } finally {
      if (_didIteratorError2) {
        throw _iteratorError2;
      }
    }
  }

  var _iteratorNormalCompletion3 = true;
  var _didIteratorError3 = false;
  var _iteratorError3 = undefined;

  try {
    for (var _iterator3 = _getIterator(_Object$entries(options.sagas || {})), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
      var _ref7 = _step3.value;

      var _ref8 = _slicedToArray(_ref7, 2);

      var name = _ref8[0];
      var saga = _ref8[1];

      sagas[name] = initializeAdditionalSaga(saga, selectors, actions, name);
    }
  } catch (err) {
    _didIteratorError3 = true;
    _iteratorError3 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion3 && _iterator3.return) {
        _iterator3.return();
      }
    } finally {
      if (_didIteratorError3) {
        throw _iteratorError3;
      }
    }
  }

  return _Object$assign((_Object$assign2 = {}, _defineProperty(_Object$assign2, moduleName, handleActions(reducerMap, initialState)), _defineProperty(_Object$assign2, 'sagas', sagas), _Object$assign2), actionCreators, actions, selectors);
};

export var createModule = function createModule(moduleName, definitions) {
  var initialState = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  var options = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};
  return createModuleWithApp(moduleName, definitions, initialState, options);
};
export var createApp = function createApp(appName) {
  return function (moduleName, definitions) {
    var initialState = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
    var options = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};
    return createModuleWithApp(moduleName, definitions, initialState, options, appName);
  };
};