'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createApp = exports.createModule = undefined;

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _entries = require('babel-runtime/core-js/object/entries');

var _entries2 = _interopRequireDefault(_entries);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _camelCase = require('redux-actions/lib/camelCase');

var _camelCase2 = _interopRequireDefault(_camelCase);

var _reduxActions = require('redux-actions');

var _effects = require('redux-saga/effects');

var _reselect = require('reselect');

var _utils = require('./utils');

var _enhance2 = require('./enhance');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var initializeSelectors = function initializeSelectors(moduleName, factory) {
  var selectors = { selectModule: function selectModule(state) {
      return state[moduleName];
    } };
  if (factory && !(0, _utils.isNormalFunction)(factory)) {
    throw new Error('Invalid option selectorFactory: It must be a function that returns selector definitions.');
  }
  if ((0, _utils.isNormalFunction)(factory)) {
    (0, _assign2.default)(selectors, factory({ selectModule: selectors.selectModule, createSelector: _reselect.createSelector, defaultMemoize: _reselect.defaultMemoize, createSelectorCreator: _reselect.createSelectorCreator, createStructuredSelector: _reselect.createStructuredSelector }));
  }

  var _loop = function _loop(name, fn) {
    if (!(0, _utils.isNormalFunction)(fn)) throw new Error('Invalid selector entry ' + name + ': It must be a function.');
    fn.effect = function () {
      for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
        args[_key] = arguments[_key];
      }

      return _effects.select.apply(undefined, [fn].concat(args));
    };
  };

  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = (0, _getIterator3.default)((0, _entries2.default)(selectors)), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var _ref = _step.value;

      var _ref2 = (0, _slicedToArray3.default)(_ref, 2);

      var name = _ref2[0];
      var fn = _ref2[1];

      _loop(name, fn);
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  return selectors;
};

var thunkifyMainGeneratorFunction = function thunkifyMainGeneratorFunction(saga) {
  return function (_ref3) {
    var type = _ref3.type,
        takeEvery = _ref3.takeEvery;
    return takeEvery(type, saga);
  };
};
var thunkifyAdditionalGeneratorFunction = function thunkifyAdditionalGeneratorFunction(saga) {
  return function (_ref4) {
    var fork = _ref4.fork;
    return fork(saga);
  };
};

var initializeMainSaga = function initializeMainSaga(saga, selectors, type, onError) {
  var errLabels = ['Invalid saga for ' + type, 'takeEvery'];
  return initializeSaga(thunkifyMainGeneratorFunction, saga, selectors, errLabels, { type: type }, onError);
};
var initializeAdditionalSaga = function initializeAdditionalSaga(saga, selectors, types, name) {
  var errLabels = ['Invalid additional saga ' + name, 'fork'];
  return initializeSaga(thunkifyAdditionalGeneratorFunction, saga, selectors, errLabels, types);
};

var initializeSaga = function initializeSaga(thunkify, saga, selectors, errLabels, types, onError) {
  if (typeof saga === 'function') {
    saga = ((0, _utils.isGeneratorFunction)(saga) ? thunkify(saga) : saga)((0, _assign2.default)({}, types, (0, _utils.mapValues)(_enhance2.enhancibleForkerThunks, function (thunk) {
      return thunk(onError);
    }), {
      enhance: function enhance(saga) {
        return (0, _enhance2.enhance)(saga, onError);
      }
    }, selectors));
  }
  if ((0, _utils.isGeneratorFunction)(saga)) return _enhance2.enhancedForkers.fork(saga);
  if ((0, _utils.isForkEffect)(saga)) return saga;
  throw new Error(errLabels[0] + ': It must be specified as one of them: \n' + ('  - generator function (=> automatically invoked by enhanced ' + errLabels[1] + '())\n') + '  - thunk that returns generator function (=> automatically invoked by enhanced fork())\n' + '  - thunk that returns redux-saga FORK effect\n' + '  - redux-saga FORK effect');
};

var createModuleWithApp = function createModuleWithApp(moduleName, definitions, initialState, options, appName) {
  var _Object$assign2;

  var prefix = appName ? '@@' + appName + '/' : '';
  var reducerMap = {};
  var actions = {};
  var actionCreators = {};
  var sagas = {};
  var selectors = initializeSelectors(moduleName, options.selectorFactory);

  var _iteratorNormalCompletion2 = true;
  var _didIteratorError2 = false;
  var _iteratorError2 = undefined;

  try {
    for (var _iterator2 = (0, _getIterator3.default)((0, _entries2.default)(definitions)), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
      var _ref5 = _step2.value;

      var _ref6 = (0, _slicedToArray3.default)(_ref5, 2);

      var type = _ref6[0];
      var definition = _ref6[1];


      var actionType = '' + prefix + moduleName + '/' + type;
      var camelType = (0, _camelCase2.default)(type);

      var _ref9 = typeof definition === 'function' ? { reducer: definition } : definition,
          creator = _ref9.creator,
          reducer = _ref9.reducer,
          saga = _ref9.saga,
          onError = _ref9.onError;

      reducer && (reducerMap[actionType] = reducer);
      actions[type] = actionType;
      actionCreators[camelType] = _reduxActions.createAction.apply(undefined, [actionType].concat((0, _toConsumableArray3.default)(Array.isArray(creator) ? creator : [creator])));
      saga && (sagas[camelType] = initializeMainSaga(saga, selectors, actionType, onError));
    }
  } catch (err) {
    _didIteratorError2 = true;
    _iteratorError2 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion2 && _iterator2.return) {
        _iterator2.return();
      }
    } finally {
      if (_didIteratorError2) {
        throw _iteratorError2;
      }
    }
  }

  var _iteratorNormalCompletion3 = true;
  var _didIteratorError3 = false;
  var _iteratorError3 = undefined;

  try {
    for (var _iterator3 = (0, _getIterator3.default)((0, _entries2.default)(options.sagas || {})), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
      var _ref7 = _step3.value;

      var _ref8 = (0, _slicedToArray3.default)(_ref7, 2);

      var name = _ref8[0];
      var saga = _ref8[1];

      sagas[name] = initializeAdditionalSaga(saga, selectors, actions, name);
    }
  } catch (err) {
    _didIteratorError3 = true;
    _iteratorError3 = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion3 && _iterator3.return) {
        _iterator3.return();
      }
    } finally {
      if (_didIteratorError3) {
        throw _iteratorError3;
      }
    }
  }

  return (0, _assign2.default)((_Object$assign2 = {}, (0, _defineProperty3.default)(_Object$assign2, moduleName, (0, _reduxActions.handleActions)(reducerMap, initialState)), (0, _defineProperty3.default)(_Object$assign2, 'sagas', sagas), _Object$assign2), actionCreators, actions, selectors);
};

var createModule = exports.createModule = function createModule(moduleName, definitions) {
  var initialState = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  var options = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};
  return createModuleWithApp(moduleName, definitions, initialState, options);
};
var createApp = exports.createApp = function createApp(appName) {
  return function (moduleName, definitions) {
    var initialState = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
    var options = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : {};
    return createModuleWithApp(moduleName, definitions, initialState, options, appName);
  };
};